"""
=======
Resolwe
=======

Open source enterprise dataflow engine in Django.

"""
from __future__ import absolute_import, division, print_function, unicode_literals

from . import flow

from resolwe.__about__ import (
    __author__, __copyright__, __email__, __license__, __summary__, __title__,
    __url__, __version__,
)
